﻿using System.Windows;
using System.Windows.Controls;
using CommunityToolkit.Mvvm.ComponentModel;
using LibS7Adv;
using CtrEditor.Simulacion;
using CtrEditor.FuncionesBase;
using System.ComponentModel;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucGuia.xaml
    /// </summary>
    public partial class osGuia : osBase, IosBase
    {
        private simGuia SimGeometria;

        public static string NombreClase()
        {
            return "Guía";
        }
        private string nombre = NombreClase();

        [property: Category("Identificación")]
        [property: Description("Nombre identificativo del objeto")]
        [property: Name("Nombre")]
        public override string Nombre
        {
            get => nombre;
            set => SetProperty(ref nombre, value);
        }

        [ObservableProperty]
        [property: Category("Configuración")]
        [property: Description("Alto de la guía en metros")]
        [property: Name("Alto de la Guía")]
        public float altoGuia;

        private void ActualizarGeometrias()
        {
            if (_visualRepresentation is ucGuia uc)
                UpdateOrCreateLine(SimGeometria, uc.Guia);
        }

        public override void OnMoveResizeRotate()
        {
            ActualizarGeometrias();
        }

        public osGuia()
        {
            Ancho = 1;
            AltoGuia = 0.03f;
        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }
        public override void UpdateControl(int elapsedMilliseconds)
        {
        }
        public override void UpdateGeometryStep()
        {
        }
        public override void UpdatePLC(PLCViewModel plc, int elapsedMilliseconds) { }
        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            base.ucLoaded();

            if (_visualRepresentation is ucGuia uc)
                SimGeometria = AddLine(simulationManager, uc.Guia);
        }
        public override void ucUnLoaded()
        {
            // El UserControl se esta eliminando
            // eliminar el objeto de simulacion
            simulationManager?.Remove(SimGeometria);
        }


    }

    public partial class ucGuia : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }
        public int zIndex_fromFrames { get; set; }

        public ucGuia()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }
        public void Highlight(bool State) { }
        public ZIndexEnum ZIndex_Base()
        {
            return ZIndexEnum.Guias;
        }

    }
}
